@@ -3,6 +3,7 @@ from __future__ import unicode_literals
 
 import logging
 import os
+import re
 import subprocess
 import sys
 
@@ -36,14 +37,19 @@ def _hook_msg_start(hook, verbose):
     )
 
 
-def get_changed_files(new, old):
-    return cmd_output(
-        'git', 'diff', '--no-ext-diff', '--name-only',
-        '{}...{}'.format(old, new),
-    )[1].splitlines()
+def _filter_by_include_exclude(filenames, include, exclude):
+    include_re, exclude_re = re.compile(include), re.compile(exclude)
+    return {
+        filename for filename in filenames
+        if (
+            include_re.search(filename) and
+            not exclude_re.search(filename) and
+            os.path.lexists(filename)
+        )
+    }
 
 
-def filter_filenames_by_types(filenames, types, exclude_types):
+def _filter_by_types(filenames, types, exclude_types):
     types, exclude_types = frozenset(types), frozenset(exclude_types)
     ret = []
     for filename in filenames:
@@ -53,34 +59,15 @@ def filter_filenames_by_types(filenames, types, exclude_types):
     return tuple(ret)
 
 
-def get_filenames(args, include_expr, exclude_expr):
-    if args.origin and args.source:
-        getter = git.get_files_matching(
-            lambda: get_changed_files(args.origin, args.source),
-        )
-    elif args.hook_stage == 'commit-msg':
-        def getter(*_):
-            return (args.commit_msg_filename,)
-    elif args.files:
-        getter = git.get_files_matching(lambda: args.files)
-    elif args.all_files:
-        getter = git.get_all_files_matching
-    elif git.is_in_merge_conflict():
-        getter = git.get_conflicted_files_matching
-    else:
-        getter = git.get_staged_files_matching
-    return getter(include_expr, exclude_expr)
-
-
 SKIPPED = 'Skipped'
 NO_FILES = '(no files to check)'
 
 
-def _run_single_hook(hook, repo, args, skips, cols):
-    filenames = get_filenames(args, hook['files'], hook['exclude'])
-    filenames = filter_filenames_by_types(
-        filenames, hook['types'], hook['exclude_types'],
-    )
+def _run_single_hook(filenames, hook, repo, args, skips, cols):
+    include, exclude = hook['files'], hook['exclude']
+    filenames = _filter_by_include_exclude(filenames, include, exclude)
+    types, exclude_types = hook['types'], hook['exclude_types']
+    filenames = _filter_by_types(filenames, types, exclude_types)
     if hook['id'] in skips:
         output.write(get_hook_message(
             _hook_msg_start(hook, args.verbose),
@@ -176,13 +163,32 @@ def _compute_cols(hooks, verbose):
     return max(cols, 80)
 
 
-def _run_hooks(repo_hooks, args, environ):
+def _all_filenames(args):
+    if args.origin and args.source:
+        return git.get_changed_files(args.origin, args.source)
+    elif args.hook_stage == 'commit-msg':
+        return (args.commit_msg_filename,)
+    elif args.files:
+        return args.files
+    elif args.all_files:
+        return git.get_all_files()
+    elif git.is_in_merge_conflict():
+        return git.get_conflicted_files()
+    else:
+        return git.get_staged_files()
+
+
+def _run_hooks(config, repo_hooks, args, environ):
     """Actually run the hooks."""
     skips = _get_skips(environ)
     cols = _compute_cols([hook for _, hook in repo_hooks], args.verbose)
+    filenames = _all_filenames(args)
+    filenames = _filter_by_include_exclude(filenames, '', config['exclude'])
     retval = 0
     for repo, hook in repo_hooks:
-        retval |= _run_single_hook(hook, repo, args, skips, cols)
+        retval |= _run_single_hook(filenames, hook, repo, args, skips, cols)
+        if retval and config['fail_fast']:
+            break
     if (
             retval and
             args.show_diff_on_failure and
@@ -199,17 +205,14 @@ def get_repo_hooks(runner):
             yield (repo, hook)
 
 
-def _has_unmerged_paths(runner):
-    _, stdout, _ = runner.cmd_runner.run(['git', 'ls-files', '--unmerged'])
+def _has_unmerged_paths():
+    _, stdout, _ = cmd_output('git', 'ls-files', '--unmerged')
     return bool(stdout.strip())
 
 
 def _has_unstaged_config(runner):
-    retcode, _, _ = runner.cmd_runner.run(
-        (
-            'git', 'diff', '--no-ext-diff', '--exit-code',
-            runner.config_file_path,
-        ),
+    retcode, _, _ = cmd_output(
+        'git', 'diff', '--no-ext-diff', '--exit-code', runner.config_file_path,
         retcode=None,
     )
     # be explicit, other git errors don't mean it has an unstaged config.
@@ -217,30 +220,21 @@ def _has_unstaged_config(runner):
 
 
 def run(runner, args, environ=os.environ):
-    no_stash = args.no_stash or args.all_files or bool(args.files)
+    no_stash = args.all_files or bool(args.files)
 
     # Check if we have unresolved merge conflict files and fail fast.
-    if _has_unmerged_paths(runner):
+    if _has_unmerged_paths():
         logger.error('Unmerged files.  Resolve before committing.')
         return 1
     if bool(args.source) != bool(args.origin):
         logger.error('Specify both --origin and --source.')
         return 1
     if _has_unstaged_config(runner) and not no_stash:
-        if args.allow_unstaged_config:
-            logger.warn(
-                'You have an unstaged config file and have specified the '
-                '--allow-unstaged-config option.\n'
-                'Note that your config will be stashed before the config is '
-                'parsed unless --no-stash is specified.',
-            )
-        else:
-            logger.error(
-                'Your .pre-commit-config.yaml is unstaged.\n'
-                '`git add .pre-commit-config.yaml` to fix this.\n'
-                'Run pre-commit with --allow-unstaged-config to silence this.',
-            )
-            return 1
+        logger.error(
+            'Your .pre-commit-config.yaml is unstaged.\n'
+            '`git add .pre-commit-config.yaml` to fix this.',
+        )
+        return 1
 
     # Expose origin / source as environment variables for hooks to consume
     if args.origin and args.source:
@@ -250,7 +244,7 @@ def run(runner, args, environ=os.environ):
     if no_stash:
         ctx = noop_context()
     else:
-        ctx = staged_files_only(runner.cmd_runner)
+        ctx = staged_files_only(runner.store.directory)
 
     with ctx:
         repo_hooks = list(get_repo_hooks(runner))
@@ -270,4 +264,4 @@ def run(runner, args, environ=os.environ):
             if not hook['stages'] or args.hook_stage in hook['stages']
         ]
 
-        return _run_hooks(repo_hooks, args, environ)
+        return _run_hooks(runner.config, repo_hooks, args, environ)
