@@ -37,14 +37,18 @@ class Runner(object):
     def config_file_path(self):
         return os.path.join(self.git_root, self.config_file)
 
+    @cached_property
+    def config(self):
+        return load_config(self.config_file_path)
+
     @cached_property
     def repositories(self):
         """Returns a tuple of the configured repositories."""
-        config = load_config(self.config_file_path)
-        repositories = tuple(Repository.create(x, self.store) for x in config)
-        for repository in repositories:
-            repository.require_installed()
-        return repositories
+        repos = self.config['repos']
+        repos = tuple(Repository.create(x, self.store) for x in repos)
+        for repo in repos:
+            repo.require_installed()
+        return repos
 
     def get_hook_path(self, hook_type):
         return os.path.join(self.git_dir, 'hooks', hook_type)
@@ -57,11 +61,6 @@ class Runner(object):
     def pre_push_path(self):
         return self.get_hook_path('pre-push')
 
-    @cached_property
-    def cmd_runner(self):
-        # TODO: remove this and inline runner.store.cmd_runner
-        return self.store.cmd_runner
-
     @cached_property
     def store(self):
         return Store()
